<?php

require_once('../config.php');
require_once('lib/php-jwt-main/vendor/autoload.php');

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

use \Firebase\JWT\JWT;


define('DB_SERVER', $db_host);
define('DB_USERNAME', $db_user);
define('DB_PASSWORD', $db_password);
define('DB_NAME', $db_name);

$app_url = 'https://app.giotheapp.com';

/* Attempt to connect to MySQL database */
$link = mysqli_connect(DB_SERVER, DB_USERNAME, DB_PASSWORD, DB_NAME);

// Check connection
if($link === false){
    die("ERROR: Could not connect. " . mysqli_connect_error());
}


function random_string($length = 32) {
    return bin2hex(random_bytes($length / 2));
}

function encryptPassword($password) {
    // Generate a password hash with a random salt
    $hashedPassword = password_hash($password, PASSWORD_DEFAULT);

    $hashedPassword = $password;

    if ($hashedPassword === false) {
        // Handle error, such as inability to generate hash
        return false;
    }

    return $hashedPassword;
}


function isEmailDuplicate($email, $link) {
    $sql = "SELECT id FROM app_users WHERE email = ?";
    $stmt = $link->prepare($sql);
    $stmt->bind_param('s', $email);
    $stmt->execute();
    $stmt->store_result();

    return $stmt->num_rows > 0;
}

// Function to authenticate user
function authenticateUser($email, $password) {
    global $link;

    // Prepare SQL statement to fetch user with given email
    $sql = "SELECT * FROM app_users WHERE email = ?";
    $stmt = $link->prepare($sql);
    $stmt->bind_param('s', $email);
    $stmt->execute();
    $result = $stmt->get_result();

    // Check if user exists
    if ($result->num_rows === 0) {
        return false; // User does not exist
    }

    // Fetch user data
    $user = $result->fetch_assoc();

    $hashedPassword = encryptPassword($password);

    // Verify password
    if ($hashedPassword==$user['password']) {
      // Determine the table based on user type

          $token = generateAndStoreToken($link, $user['id']);

          $user['token']=$token;
          $user['image']='https://app.giotheapp.com/webservices/uploads/default-user.png';
if($user['user_type'] == 'supplier') {
    $user['supplier_id'] = 0;
} elseif($user['user_type'] == 'restaurant') {
    $user['supplier_id'] = gnr($link, 'users_relation', 'restaurant_id', $user['id'], 'supplier_id');
} else {
    $user['supplier_id'] = gnr($link, 'users_relation', 'store_id', $user['id'], 'supplier_id');
}
          unset($user['password']);

          return $user; // Password is correct

    } else {
        return false; // Password is incorrect
    }

}

function validateEmail($email) {
    return filter_var($email, FILTER_VALIDATE_EMAIL);
}

function generateAndStoreToken($conn, $userId) {
    // Generate a JWT token as shown earlier
    $jwtSecret = 'BasePlan';
    $userData = [
      'timestamp' => time(),  // This will be different every second
      'user_id' => $userId, // Unique user identifier
        // Other user claims...
    ];
    $token = JWT::encode($userData, $jwtSecret, 'HS256');

    // Set token expiration (e.g., 1 hour from now)
    $expiration = date('Y-m-d H:i:s', strtotime('+1 week'));

    // Store the token in the database
    $sql = "INSERT INTO user_tokens (user_id, token, expiration) VALUES (?, ?, ?)";
    $stmt = $conn->prepare($sql);
    $stmt->bind_param('iss', $userId, $token, $expiration);

    if ($stmt->execute()) {
        return $token; // Return the generated token
    } else {
        // Token storage failed, handle the error
        return false;
    }
}


function validateToken($conn, $token) {
    // Check if the token exists in the database
    $sql = "SELECT user_id, expiration FROM user_tokens WHERE token = ?";
    $stmt = $conn->prepare($sql);
    $stmt->bind_param('s', $token);
    $stmt->execute();
    $result = $stmt->get_result();

    if ($result->num_rows === 1) {
        $row = $result->fetch_assoc();
        $userId = $row['user_id'];
        $expiration = $row['expiration'];

        // Check if the token is expired
        $currentDateTime = date('Y-m-d H:i:s');
        if ($expiration > $currentDateTime) {
            // Token is valid, return the associated user ID
            return $userId;
        } else {
            // Token has expired, handle accordingly
            return false;
        }
    } else {
        // Token not found in the database or invalid
        return false;
    }
}

// Function to get the user ID from the token
function getUserIdFromToken($token, $link) {
    // Here, you'd implement logic to validate the token and fetch the associated user ID.


    // The current date and time, formatted for SQL
    $currentDateTime = date('Y-m-d H:i:s');

    $sql = "SELECT * FROM `user_tokens` WHERE token = ? AND expiration > ?";
    $stmt = $link->prepare($sql);
    $stmt->bind_param('ss', $token, $currentDateTime);
    $stmt->execute();
    $result = $stmt->get_result();
    $row = $result->fetch_assoc();

    return $row['user_id'] ?? null;
}

function generateConfirmationCode() {
    $characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $characters = '012345689';
    $code = '';
    for ($i = 0; $i < 6; $i++) {
        $code .= $characters[mt_rand(0, strlen($characters) - 1)];
    }
    return $code;
}

function checkDuplicateEmail($email, $link) {
    $sql = "SELECT id FROM app_users WHERE email = ?";
    $stmt = $link->prepare($sql);
    $stmt->bind_param('s', $email);
    $stmt->execute();
    $result = $stmt->get_result();
    if ($result->num_rows > 0) {
        return true; // Duplicate found
    } else {
        return false; // No duplicate
    }
}

// Function to generate a random password
function generateRandomPassword($length = 9) {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $characters = '0123456789';
    $password = '';
    for ($i = 0; $i < $length; $i++) {
        $password .= $characters[rand(0, strlen($characters) - 1)];
    }
    return $password;
}

function handleImageUpload() {
    if (!isset($_FILES['image']) || $_FILES['image']['error'] !== UPLOAD_ERR_OK) {
        return null; // No image uploaded or error occurred
    }

    // Generate a random filename to avoid collisions
    $randomFilename = uniqid('image_') . '.' . pathinfo($_FILES['image']['name'], PATHINFO_EXTENSION);

    // Move the uploaded file to the uploads directory
    $uploadDir = 'uploads/';
    $destination = $uploadDir . $randomFilename;
    move_uploaded_file($_FILES['image']['tmp_name'], $destination);

    return $destination; // Return the path to the uploaded image
}

function gnr($conn, $table, $columnToSelect, $valueToMatch, $columnToReturn) {
    $sql = "SELECT $columnToReturn
            FROM $table
            WHERE $columnToSelect = ?";


    // Prepare the statement
    $stmt = $conn->prepare($sql);

    if (!$stmt) {
        return "Error in SQL query: " . $conn->error;
    }

    // Bind the parameter
    $stmt->bind_param("s", $valueToMatch);

    // Execute the statement
    if (!$stmt->execute()) {
        return "Error executing SQL query: " . $stmt->error;
    }

    // Bind the result to a variable
    $stmt->bind_result($columnValue);

    // Check the number of rows selected
    $stmt->store_result();
    $rowCount = $stmt->num_rows;

    // echo 'rows: '.$rowCount;

    if ($rowCount > 0) {
        // Fetch the first row
        $stmt->fetch();
        return $columnValue;
    }

    // If no rows were found, return null or handle the situation as desired
    // echo 'null';
}
 ?>
