<?php

require_once 'functions.php'; // Include database connection and utility functions

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $userData = json_decode(file_get_contents('php://input'), true);

    if (empty($userData['first_name']) || empty($userData['business_name']) ||
        empty($userData['email']) || empty($userData['password']) ||
        empty($userData['user_type'])) {
        http_response_code(400);
        echo json_encode(['error' => 'Missing required data']);
        exit;
    }

    $first_name = $userData['first_name'];
    $last_name = $userData['last_name'] ?? '';
    $user_type = $userData['user_type'];
    $business_name = $userData['business_name'];
    $address_1 = $userData['address_1'] ?? '';
    $address_2 = $userData['address_2'] ?? '';
    $city = $userData['city'] ?? '';
    $zip_code = $userData['zip_code'] ?? '';
    $email = $userData['email'];
    $phone = $userData['phone'] ?? '';
    $password = $userData['password'];
    $status = 'active';

    if (!validateEmail($email)) {
        http_response_code(400);
        echo json_encode(['error' => 'Invalid email format']);
        exit;
    }
    if (isEmailDuplicate($email, $link)) {
        http_response_code(409); // Email already exists
        echo json_encode(['error' => 'Email already in use']);
        exit;
    }

    $passwordHashed = encryptPassword($password);

    if ($passwordHashed === false) {
        http_response_code(500);
        echo json_encode(['error' => 'Could not hash password']);
        exit;
    }

    if ($user_type === 'restaurant' || $user_type === 'shop') {
        if (empty($userData['supplier_id'])) {
            http_response_code(400);
            echo json_encode(['error' => 'Supplier ID is required for restaurant or shop']);
            exit;
        }
    }


    $sql = "INSERT INTO app_users (first_name, last_name,  user_type, business_name, address_1, address_2, city, zip_code, email, phone, password,status)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    $stmt = $link->prepare($sql);
    $stmt->bind_param('ssssssssssss', $first_name, $last_name,  $user_type, $business_name, $address_1, $address_2, $city, $zip_code, $email, $phone, $passwordHashed,$status);

    if ($stmt->execute()) {
        $userId = $stmt->insert_id;

        if ($user_type === 'restaurant' || $user_type === 'shop') {
            if (empty($userData['supplier_id'])) {
                http_response_code(400);
                echo json_encode(['error' => 'Supplier ID is required for restaurant or shop']);
                exit;
            }

            $supplier_id = $userData['supplier_id'];
            $column = ($user_type === 'restaurant') ? 'restaurant_id' : 'store_id';

            $sqlRelation = "INSERT INTO users_relation ($column, supplier_id) VALUES (?, ?)";
            $stmtRelation = $link->prepare($sqlRelation);
            $stmtRelation->bind_param('ii', $userId, $supplier_id);

            if (!$stmtRelation->execute()) {
                http_response_code(500);
                echo json_encode(['error' => 'Could not create user relation']);
                exit;
            }
        }

        $user = authenticateUser($email, $password);

        http_response_code(201);
        echo json_encode(['message' => 'User registered successfully', 'data' => $user]);
    } else {
        http_response_code(500); // Internal Server Error
        echo json_encode(['error' => 'Could not register user']);
    }
} else {
    http_response_code(405);
    echo json_encode(['error' => 'Method not allowed']);
}
?>
